@@ -19,32 +19,32 @@ use macros::create_broadcast_role_short::CreateBroadcastRoleShortMacroInput;
 use macros::create_normal_role_short::CreateNormalRoleShortMacroInput;
 use macros::multiple::broadcast_cancel::BroadcastCancelMacroInput;
 use macros::multiple::checking::CheckingInput;
-use macros::multiple::choose_mpst_create_multi_to_all::ChooseMultiCreateToAllMacroInput;
-use macros::multiple::choose_mpst_multi_cancel_to_all::ChooseTypeMultiCancelToAllMacroInput;
-use macros::multiple::choose_mpst_multi_http_to_all::ChooseTypeMultiHttpToAllMacroInput;
-use macros::multiple::choose_mpst_multi_to_all::ChooseMultiToAllMacroInput;
-use macros::multiple::close_mpst::CloseMpstMacroInput;
-use macros::multiple::close_mpst_cancel::CloseMpstCancelMacroInput;
-use macros::multiple::close_mpst_check_cancel::CloseMpstCheckCancelMacroInput;
-use macros::multiple::create_choose_mpst_session_multi_left::ChooseTypeMultiLeftMacroInput;
-use macros::multiple::create_choose_mpst_session_multi_right::ChooseTypeMultiRightMacroInput;
-use macros::multiple::create_choose_type_multi::ChooseTypeMultiMacroInput;
-use macros::multiple::create_fn_choose_mpst_cancel_multi_to_all_bundle::ChooseTypeCancelMultiToAllBundleMacroInput;
-use macros::multiple::create_fn_choose_mpst_multi_to_all_bundle::ChooseTypeMultiToAllBundleMacroInput;
+use macros::multiple::choose::choose_mpst_create_multi_to_all::ChooseMultiCreateToAllMacroInput;
+use macros::multiple::choose::choose_mpst_multi_cancel_to_all::ChooseTypeMultiCancelToAllMacroInput;
+use macros::multiple::choose::choose_mpst_multi_http_to_all::ChooseTypeMultiHttpToAllMacroInput;
+use macros::multiple::choose::choose_mpst_multi_to_all::ChooseMultiToAllMacroInput;
+use macros::multiple::choose::create_choose_mpst_session_multi_left::ChooseTypeMultiLeftMacroInput;
+use macros::multiple::choose::create_choose_mpst_session_multi_right::ChooseTypeMultiRightMacroInput;
+use macros::multiple::choose::create_choose_type_multi::ChooseTypeMultiMacroInput;
+use macros::multiple::choose::create_fn_choose_mpst_cancel_multi_to_all_bundle::ChooseTypeCancelMultiToAllBundleMacroInput;
+use macros::multiple::choose::create_fn_choose_mpst_multi_to_all_bundle::ChooseTypeMultiToAllBundleMacroInput;
+use macros::multiple::close::close_mpst::CloseMpstMacroInput;
+use macros::multiple::close::close_mpst_cancel::CloseMpstCancelMacroInput;
+use macros::multiple::close::close_mpst_check_cancel::CloseMpstCheckCancelMacroInput;
 use macros::multiple::create_meshedchannels::CreateMeshedChannelsMacroInput;
-use macros::multiple::create_offer_mpst_session_multi::OfferMPSTSessionMultiMacroInput;
-use macros::multiple::create_offer_type_multi::OfferTypeMultiMacroInput;
-use macros::multiple::create_recv_http_session::CreateRecvHttpSessionMacroInput;
-use macros::multiple::create_recv_mpst_all_session::CreateRecvMPSTAllSessionMacroInput;
-use macros::multiple::create_recv_mpst_session::CreateRecvMPSTSessionMacroInput;
-use macros::multiple::create_send_check_cancel::CreateSendCheckCancelMacroInput;
-use macros::multiple::create_send_http_session::CreateSendHttpSessionMacroInput;
-use macros::multiple::create_send_mpst_cancel::CreateSendMPSTCancelMacroInput;
-use macros::multiple::create_send_mpst_session::CreateSendMPSTSessionMacroInput;
 use macros::multiple::fork_mpst_multi::ForkMPSTMultiMacroInput;
-use macros::multiple::recv_mpst::RecvMPSTMacroInput;
-use macros::multiple::send_cancel::SendCancelMacroInput;
-use macros::multiple::send_mpst::SendMPSTMacroInput;
+use macros::multiple::offer::create_offer_mpst_session_multi::OfferMPSTSessionMultiMacroInput;
+use macros::multiple::offer::create_offer_type_multi::OfferTypeMultiMacroInput;
+use macros::multiple::recv::create_recv_http_session::CreateRecvHttpSessionMacroInput;
+use macros::multiple::recv::create_recv_mpst_all_session::CreateRecvMPSTAllSessionMacroInput;
+use macros::multiple::recv::create_recv_mpst_session::CreateRecvMPSTSessionMacroInput;
+use macros::multiple::recv::recv_mpst::RecvMPSTMacroInput;
+use macros::multiple::send::create_send_check_cancel::CreateSendCheckCancelMacroInput;
+use macros::multiple::send::create_send_http_session::CreateSendHttpSessionMacroInput;
+use macros::multiple::send::create_send_mpst_cancel::CreateSendMPSTCancelMacroInput;
+use macros::multiple::send::create_send_mpst_session::CreateSendMPSTSessionMacroInput;
+use macros::multiple::send::send_cancel::SendCancelMacroInput;
+use macros::multiple::send::send_mpst::SendMPSTMacroInput;
 
 //////////////////////////////////////
 
